@@ -1,21 +1,14 @@
 import logging
 
-from wsme.exc import UnknownFunction, MissingArgument, UnknownArgument
-from wsme.protocols import CallContext
+from wsme.exc import ClientSideError, UnknownArgument
+from wsme.protocols import CallContext, Protocol
+from wsme.protocols.commons import from_params
+from wsme.types import Unset
 
 log = logging.getLogger(__name__)
 
 
-class RestProtocol(object):
-    name = None
-    dataformat = None
-    content_types = []
-
-    def accept(self, request):
-        if request.path.endswith('.' + self.dataformat):
-            return True
-        return request.headers.get('Content-Type') in self.content_types
-
+class RestProtocol(Protocol):
     def iter_calls(self, request):
         yield CallContext(request)
 
@@ -33,7 +26,16 @@ class RestProtocol(object):
     def read_arguments(self, context):
         request = context.request
         funcdef = context.funcdef
-        if len(request.params) and request.body:
+
+        if 'Content-Type' in request.headers \
+                and "application/x-www-form-urlencoded" in \
+                    request.headers['Content-Type']:
+            # The params were read from the body, ignoring the body then
+            pass
+        elif len(request.params) and request.body:
+            log.warning("The request has both a body and params.")
+            log.debug("Params: %s" % request.params)
+            log.debug("Body: %s" % request.body)
             raise ClientSideError(
                 "Cannot read parameters from both a body and GET/POST params")
 
@@ -42,9 +44,18 @@ class RestProtocol(object):
             body = request.params['body']
 
         if body is None and len(request.params):
-            parsed_args = {}
-            for key, value in request.params.items():
-                parsed_args[key] = self.parse_arg(key, value)
+            kw = {}
+            hit_paths = set()
+            for argdef in funcdef.arguments:
+                value = from_params(
+                    argdef.datatype, request.params, argdef.name, hit_paths)
+                if value is not Unset:
+                    kw[argdef.name] = value
+            paths = set(request.params.keys())
+            unknown_paths = paths - hit_paths
+            if unknown_paths:
+                raise UnknownArgument(', '.join(unknown_paths))
+            return kw
         else:
             if body is None:
                 body = request.body
